import { PAGLayer } from './pag-layer';
import type { PAGImage } from './pag-image';
import type { PAGVideoRange } from './types';
export declare class PAGImageLayer extends PAGLayer {
    /**
     * Make a empty PAGImageLayer with specified size.
     */
    static make(width: number, height: number, duration: number): PAGImageLayer;
    /**
     * Returns the content duration in microseconds, which indicates the minimal length required for
     * replacement.
     */
    contentDuration(): number;
    /**
     * Returns the time ranges of the source video for replacement.
     */
    getVideoRanges(): PAGVideoRange[];
    /**
     * [Deprecated]
     * Replace the original image content with the specified PAGImage object.
     * Passing in null for the image parameter resets the layer to its default image content.
     * The replaceImage() method modifies all associated PAGImageLayers that have the same
     * editableIndex to this layer.
     *
     * @param image The PAGImage object to replace with.
     */
    replaceImage(pagImage: PAGImage): void;
    /**
     * Replace the original image content with the specified PAGImage object.
     * Passing in null for the image parameter resets the layer to its default image content.
     * The setImage() method only modifies the content of the calling PAGImageLayer.
     *
     * @param image The PAGImage object to replace with.
     */
    setImage(pagImage: PAGImage): void;
    /**
     * Converts the time from the PAGImageLayer's timeline to the replacement content's timeline. The
     * time is in microseconds.
     */
    layerTimeToContent(layerTime: number): number;
    /**
     * Converts the time from the replacement content's timeline to the PAGLayer's timeline. The time
     * is in microseconds.
     */
    contentTimeToLayer(contentTime: number): number;
    /**
     * The default image data of this layer, which is webp format.
     */
    imageBytes(): Uint8Array | null;
}
